// ==============================================================
//                 ORBITER MODULE: ShuttlePB
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// ShuttlePB.cpp
// Control module for ShuttlePB vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#define GEAR				1
#define CANOPY				2
#define START               3

#include <orbitersdk.h>
#include <cstdio>
#include <cmath>


#include <string>
#include "OrbiterSoundSDK35.h"

#include <stdio.h>
#include <stdlib.h>

using namespace std;


// ==============================================================
// Some vessel parameters
// ==============================================================
const double PB_FUELMASS = 10000.0;
const double PB_ISP = 50000.0;
const double PB_MAXMAINTH =0;

double Cockpit= 0;
double iop = 0;
double st = 0;
double pld = 0;
double cen = 0;
double gear_proc = 0;
double gear_procl = 0;
double exproc = 0;
//double Lights = 0;
//double switched = 0;
double aoa;
double dil = 0;
double eng_pitch = 12000;
double canopy = 0;
double propscale = 0;
double start = 0;
double full = 0.001;
double i = 0;
double i1 = 0;
double i2 = 0;
double i3 = 0;
double i4 = 0;
double i5 = 0;
double i7 = 0;
double i9 = 0;
double i1a = 0;
double i2a = 0;
double i3a = 0;
double i4a = 0;
double i5a = 0;

double power_proc = 0;

double i13 = 0;
double i14 = 0;
double i15 = 0;
double i16 = 0;
double i17 = 0;
double i18 = 0;
double i19 = 0;
double i20 = 0;
double i21 = 0;

double i25 = 0;
double i26 = 0;
double tpp = 0;


double i33 = 0;

double s1 = 0;
double s2 = 0;
double s3 = 0;
double s4 = 0;
double s5 = 0;
double ga = 0;
double diff = 0.1;
double k3;
int k4;
char k3buffer [33];
double amet = 0;
double dee = 0;
double dee33 = 0;
double propanm = 0;
double propanm2 = 0;

double refuel = 0;

//double flaps = 0;

double move = 2;
double move_down = 0.333;

UINT animactive;

UINT animactivel;

UINT animactive2;

UINT animactive3;

UINT animactive4;

UINT animactive5;


const VECTOR3 offset = {0,-0.111,0.333};

const VECTOR3 offset_prop = {-1.903685E-02,0.1247745,-2.85};

const VECTOR3 offset_propb = {-1.903685E-02,0.1247745,-3.45};

THRUSTER_HANDLE th_main;

MESHHANDLE f150;
MESHHANDLE f151;
MESHHANDLE f152;
MESHHANDLE f153;


// 1. vertical lift component (wings and body)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6);
}


// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class ShuttlePB: public VESSEL2 {
public:
	ShuttlePB (OBJHANDLE hVessel, int flightmodel);
	//	: VESSEL2 (hVessel, flightmodel) {}
	
	~ShuttlePB();

	
	void clbkSetClassCaps (FILEHANDLE cfg);
	void DefineAnimations(void);
	void Steering();
	
	void clbkPostCreation (void);
    void clbkPostStep (double simt, double simdt, double mjd);
    int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
   
    UINT anim_wheel;
       
    UINT anim_gear;
    
    UINT anim_propscale;

    UINT anim_steer;
    
    //UINT anim_flaps;

    UINT anim_dial44;

    UINT anim_canopy;

    UINT anim_gearlever;
    
    UINT anim_rudder;
    UINT anim_elevator;
    UINT anim_laileron;
    UINT anim_raileron;
    
    UINT anim_propelar;
    
    UINT anim_wheel0;
    UINT anim_wheel1;
    UINT anim_wheel2;
    UINT anim_wheel3;
    UINT anim_wheel4;
    UINT anim_wheel5;
    
    UINT anim_dial1;
    UINT anim_dial2;
    UINT anim_dial3;

    UINT anim_bank;
    UINT anim_pitch;

    UINT anim_power;

    UINT anim_clock_s;
    UINT anim_clock_m;
    UINT anim_clock_h;
    
    UINT anim_turn;
    
    bool clbkLoadVC (int id);
    
    MGROUP_TRANSFORM *pitch1;
    
    
    int MySoundID;	


};

ShuttlePB::ShuttlePB (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel)
{
	
	// preload meshes
	
    f150                 = oapiLoadMeshGlobal ("K33\\LongEZ");
    f151                 = oapiLoadMeshGlobal ("K33\\LongEZ-Cpit");
    f152                 = oapiLoadMeshGlobal ("K33\\LongEZ-Prop");
    f153                 = oapiLoadMeshGlobal ("K33\\LongEZ-Prop-Blurred");
   
	
	DefineAnimations ();
}


void ShuttlePB::clbkPostCreation (void)
{

int MySoundID=ConnectToOrbiterSoundDLL3(GetHandle());

ReplaceStockSound3(MySoundID,"Sound\\LongEZ\\LongEZ-Eng.wav",	REPLACE_MAIN_THRUST);

RequestLoadVesselWave3(MySoundID,GEAR,"Sound\\LongEZ\\LongEZ-Gear.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,CANOPY,"Sound\\LongEZ\\LongEZ-Canopy.wav",BOTHVIEW_FADED_FAR);

RequestLoadVesselWave3(MySoundID,START,"Sound\\LongEZ\\LongEZ-Start.wav",BOTHVIEW_FADED_FAR);

SoundOptionOnOff3(MySoundID,PLAYHOVERTHRUST,FALSE);


SoundOptionOnOff3(MySoundID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINAIRCONDITIONING,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINRANDOMAMBIANCE,FALSE);

SoundOptionOnOff3(MySoundID,PLAYRADIOATC,FALSE);

SoundOptionOnOff3(MySoundID,DISPLAYTIMER,FALSE);

}


// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------

void ShuttlePB::clbkSetClassCaps (FILEHANDLE cfg)
{
	// THRUSTER_HANDLE th_main;


	// physical specs
	SetSize (5);
	SetEmptyMass (12000.0);
	SetCW (0.3, 0.3, 0.6, 0.9);
	SetWingAspect (1.7);
	SetWingEffectiveness (3.1);
	SetCrossSections (_V(5,5,15));
	SetRotDrag (_V(3.5,3.5,3.5));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-4);
		SetBankMomentScale (1e-4);
	}
	SetPMI (_V(15.5,22.1,7.7));
	SetTrimScale (0.05);
	SetCameraOffset (_V(0,0,0));
	
	SetTouchdownPoints (_V(0,-1.594001,3.712145), _V(-1.276879,-1.780029,-0.3925317), _V(1.276879,-1.780029,-0.3925317));

	// **************************** aerodynamics ***********************************

	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.5), VLiftCoeff, 20, 270, 2.266);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 20,  50, 1.5);





	CreateControlSurface (AIRCTRL_ELEVATOR,     7, 0.7, _V(   0,0,-17), AIRCTRL_AXIS_XPOS,anim_elevator);
	CreateControlSurface (AIRCTRL_RUDDER,       2, 0.5, _V(   0,2,-15), AIRCTRL_AXIS_YPOS,anim_rudder);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V( 17.8,0,-17.8), AIRCTRL_AXIS_XPOS,anim_raileron);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V(-17.8,0,-17.8), AIRCTRL_AXIS_XNEG,anim_laileron);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 5, 0.1, _V(   0,0,-15), AIRCTRL_AXIS_XPOS,anim_elevator);
    

	CreateVariableDragElement (&canopy, 3.33, _V(0,1,0));

	// propellant resources
	PROPELLANT_HANDLE hpr = CreatePropellantResource (PB_FUELMASS);
	
	
	

	// ***************** thruster definitions *******************

	
	
	th_main = CreateThruster (_V(0,0,-5), _V(0,0,1), PB_MAXMAINTH, hpr, PB_ISP);
	
	
	
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
	
	
	
	
	
 
	// visual specs
	
	AddMesh (f150);
    SetMeshVisibilityMode( AddMesh (f151,&offset), MESHVIS_VC );
    SetMeshVisibilityMode( AddMesh (f152, &offset_prop), MESHVIS_ALWAYS );
    

}

// ==============================================================
// API callback interface
// ==============================================================

void ShuttlePB::clbkPostStep (double simt, double simdt, double mjd)

{
	
	if( oapiGetTimeAcceleration() > 10.0 ) oapiSetTimeAcceleration(10);
	
	double vsAirSpeed = GetAirspeed();
	
	
	
	
    //speed and altitude control
	
	if (GetAirspeed() >= 177) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (th_main ,0);

	
}

if (GetAltitude() >= 7000) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (th_main ,0);

	
}



if (GetAirspeed() < 177 && GetAltitude() < 7000 ) 
	{
		SetCW (0.3, 0.3, 0.67, 0.92);
		SetThrusterMax0 (th_main ,3e5);

	
}

	double vsAirSpeed2 = (vsAirSpeed/1000)*3600;
    
    //Ground Steering
    
    if (GroundContact () && vsAirSpeed <=65) Steering();
    
    
    //aoa = GetAOA();
    
    //sprintf(oapiDebugString(),"anim %2.2f", aoa );
    
  
    
    SetTouchdownPoints (_V(0,-1.594001 + gear_proc,3.712145), _V(-1.276879,-1.780029,-0.3925317), _V(1.276879,-1.780029,-0.3925317));

   // Start sound
	if (vsAirSpeed <=0.333) st=0;
	
	if (vsAirSpeed >0.333) st=1;
	
	if ( vsAirSpeed <=0.333 && pld == 1 ) pld = 0;
	
	if (vsAirSpeed <0.555 && st == 1 && pld == 0) 
	{
	
	PlayVesselWave3(MySoundID,START,NOLOOP,255);
	pld = 1;
	
}

    

    //Anims 
    
    if (iop > 1)iop=0;
    
    if (vsAirSpeed > 0) {
    
	double aspeed = vsAirSpeed;
    if (iop > 1)iop=0;
    if (vsAirSpeed > 30) aspeed = 30; 
    iop = iop + (aspeed /300);
     
    if (vsAirSpeed ==0) iop=0;
    
    
    if ( GroundContact() && gear_proc < 0.05) SetAnimation (anim_wheel, iop);
    

   }
	
  
   // Gear animations


   if( animactive ) {
		if( animactive == 1 ) {
			gear_proc += 0.0025;
			if( gear_proc > 1.0 ) {
				gear_proc = 1.0;
				animactive = 0;
			}
		}
		else  {
			gear_proc -= 0.0025;
			if( gear_proc < 0.0 ) {
				gear_proc = 0.0;
				animactive = 0;
			}
		}
		SetTouchdownPoints (_V(0,-1.594001 + gear_proc,3.712145), _V(-1.276879,-1.780029,-0.3925317), _V(1.276879,-1.780029,-0.3925317));

		SetAnimation( anim_gear, gear_proc );
	}

	
	
	 // Front Gear Turn Anims1
   if (!GroundContact() && gear_proc < 1.0 && vsAirSpeed > 0.6)
   {
	   tpp = 0.5;
	   SetAnimation( anim_turn, tpp );
}

   // Front Gear Turn Anims2
   if (GroundContact() && gear_proc < 0.05 )
   {
	   
   tpp = (GetControlSurfaceLevel (AIRCTRL_RUDDER) +1) *0.5;
   if (!GroundContact())tpp = 0.5;
   SetAnimation( anim_turn, tpp );
   
}

if (GroundContact() && gear_proc > 0.05)
   {
	   tpp = 0.5;
	   SetAnimation( anim_turn, tpp );
}
	 
   
	// Canopy Animations


   if( animactive5 ) {
		if( animactive5 == 1 ) {
			canopy += 0.003;
			if( canopy > 1.0 ) {
				canopy = 1.0;
				animactive5 = 0;
			}
		}
		else  {
			canopy -= 0.003;
			if( canopy < 0.0 ) {
				canopy = 0.0;
				animactive5 = 0;
			}
		}
		SetAnimation( anim_canopy, canopy );
	}

	
   
   //Prop blurred managment
   
   if (vsAirSpeed > 1) {
	
	// SetMeshVisibilityMode( InsertMesh (f155,1,&offset), MESHVIS_VC );
     
   SetMeshVisibilityMode( InsertMesh (f153,2,&offset_propb), MESHVIS_ALWAYS );
   
}

   if (vsAirSpeed <= 1) {
	   
   SetMeshVisibilityMode( InsertMesh (f152,2,&offset_prop), MESHVIS_ALWAYS );
   
}

  
   
   // Prop Animation
   
   if (GetThrusterGroupLevel (th_main) >= 0.1 )propanm2 = 1;
   
   if (GetThrusterGroupLevel (th_main) < 0.1 && GetAirspeed() <= 0.1 )propanm2 = 0;
   
  
   
   if (GetAirspeed() > 0.33 )propanm2 = 1;
   
   if (propanm2 == 1 )
   
   {
	   if (propanm > 1)propanm=0;
	   
    propanm = propanm + 0.1;
     
       
	   SetAnimation (anim_propelar ,propanm);
	   
   }
  
   
   if (GetThrusterGroupLevel (th_main) == 0 && GetAirspeed() <= 0.33)SetAnimation (anim_propelar ,0);
   

   SetAnimation (anim_propscale ,1);
   

   // VSI Dial Animation

    aoa = GetPitch();
    
    
   i7 = ( aoa + 1) * 0.5;
   
   if ( i7 > 1 ) i7 = 1;
   if ( i7 < 0 ) i7 = 0;
   
   //sprintf(oapiDebugString(),"vsi %5.5f", aoa );

   SetAnimation (anim_dial1, i7);
   
   
   // Direction Dial Animation

   oapiGetFocusHeading(&dil);
    
   i9 = (dil / 6.25) + 0.001;
   
   //sprintf(oapiDebugString(),"hed %2.2f", dil );

   SetAnimation (anim_dial2, i9);

    // Horizon Bank Guage Animation
    
   i15 = GetBank();
   
   i16 = (i15 + 3) / 6;
   
   if (i16 < 0) i16 = 0;
   
   if (i16 > 1) i16 = 1;
   
   SetAnimation (anim_bank, i16);
   
    //sprintf(oapiDebugString(),"hed %5.5f", i15 );
    
    
    
     // Pitch Guage Animation
    
   i25 = GetPitch();
   
   i26 = (i25 + 3) / 6;
   
   if (i26 < 0) i26 = 0;
   
   if (i26 > 1) i26 = 1;
   
   SetAnimation (anim_pitch, i26);



    // Throttle animation

        power_proc = GetThrusterLevel(th_main);
	
		SetAnimation( anim_power, power_proc );
		


   // Speed Dial Animation
    
   i13 = GetAirspeed();
   
   if (i13 > 220) i13 = 220;
   
   if (i13 < 0) i13 = 0;
   
   i14 = (-i13 + 220)/220;
   
   if (i14 < 0) i14 = 0;
   
   if (i14 > 1) i14 = 1;
   
   SetAnimation (anim_dial3, i14);


   
   
   
   

   
   // Refuel
   
   if (refuel == 1)
   {
	   
	  SetFuelMass (10000.0);

	   refuel = 0;
   }

   
   // Altimeter animation
   
   
    //Anims 
    
     k3 = GetAltitude();
    
     k4 = int (k3);
      
     gcvt (k3,6,k3buffer);
     
     

   
     string stringOne(k3buffer);
     
     if (k3 >= 0 )
     {

     string is1 = stringOne.substr(0 + s2 + s3 + s4 + s5, 1) ;
       
     char const *cString1 = is1.c_str();
     
     i1a = atol(cString1);
     
     i1 = i1a * 0.1;
     
 }
 
 if (k3 >= 10 ) 
 {
	 
     string is2 = stringOne.substr(0 + s3 + s4 + s5 , 1) ;
       
     char const *cString2 = is2.c_str();
     
     i2a = atol(cString2);
     
     i2 = i2a * 0.1;
      s2 = 1;
 }
 
 if (k3 >= 100 )
 {
     string is3 = stringOne.substr(0 + s4 + s5, 1) ;
       
     char const *cString3 = is3.c_str();
     
     i3a = atol(cString3);
     
     i3 = i3a * 0.1;
     
     s3 = 1;
 }
 
 if (k3 >= 1000)
 {
     string is4 = stringOne.substr(0 + s5, 1) ;
       
     char const *cString4 = is4.c_str();
     
     i4a = atol(cString4);
     
     i4 = i4a * 0.1;
     
     s4 = 1;
 }
 
 if (k3 >= 10000)
 {
     string is5 = stringOne.substr(0, 1) ;
       
     char const *cString5 = is5.c_str();
     
     i5a = atol(cString5);
     
     i5 = i5a * 0.1;
     
     s5 = 1;
}

      if(k3<10)
      {
	      s2=0;
	      i2=0;
	      s3=0;
	      i3=0;
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
         
    if(k3<100)
      {
	     
	      s3=0;
	      i3=0;
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
    
    if(k3<1000)
      {
	      
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
   
    if(k3<10000)
      {
	     
	      s5=0;
	      i5=0;
	      
    }
    
    
   i=(k3 * 0.01)- dee;
   
   
   
   if (i>=1)
   {
	   dee = dee + 1;
   }
   
   if (i <= 0)
   {
	   dee = dee - 1;
	   
	   if (dee < 0)dee = 0;
   }
   
   
    i33=(k3 * 0.001)- dee33;
   
   
   
   if (i33>=1)
   {
	   dee33 = dee33 + 1;
   }
   
   if (i33 <= 0)
   {
	   dee33 = dee33 - 1;
	   
	   if (dee33 < 0)dee33 = 0;
   }
   
    
	//sprintf(oapiDebugString(),"force %2.2f", i1 );
	
	
	SetAnimation (anim_wheel0, i);
	SetAnimation (anim_wheel1, i33);
	//SetAnimation (anim_wheel2, i2);
	//SetAnimation (anim_wheel3, i3);
	//SetAnimation (anim_wheel4, i4);
	//SetAnimation (anim_wheel5, i5);
	
	

   
   
}  
    

//Ground Steering


void ShuttlePB::Steering()
 {

	double pos = GetControlSurfaceLevel(AIRCTRL_RUDDER);
	double lev = 3.33e6 * pos * GetAirspeed() / 10.0;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

	AddForce( _V( lev,0,0 ), _V( 0,0,0.75 ) );
	AddForce( _V( -1.0*lev,0,0 ), _V( 0,0,-0.75 ) );
	
}




// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int ShuttlePB::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events

	// if (KEYMOD_SHIFT (kstate)) {

		switch (key) {
			
			case OAPI_KEY_G: // Gear
		
		if( gear_proc > 0.5 )
					animactive = 2;
				else
					animactive = 1;

		PlayVesselWave3(MySoundID,GEAR,NOLOOP,255);
		
		
		if( gear_procl > 0.5 )
					animactivel = 2;
				else
					animactivel = 1;

			
			return 1;

		
		case OAPI_KEY_0: // Canopy
		
		if( canopy > 0.5 )
					animactive5 = 2;
				else
					animactive5 = 1;

		PlayVesselWave3(MySoundID,CANOPY,NOLOOP,255);
		
			
			return 1;

			
			
			case OAPI_KEY_J: // Refuel
		
		if ( refuel == 0)
		{
			refuel = 1;
                 
								
			return 1;
}
			else
     		{
											
			return 1;
}

			
}
	return 0;
}


// --------------------------------------------------------------
// Destructor
// --------------------------------------------------------------
ShuttlePB::~ShuttlePB ()
{
	
	delete pitch1;
	
}



void ShuttlePB::DefineAnimations(void)
{
	
	ANIMATIONCOMPONENT_HANDLE parent1;

	
	
   // Front Wheel
   //static UINT WGrp1[1] = {17};
   
   //static MGROUP_ROTATE wheel1 (0,WGrp1, 1, _V(0,-1.332981,2.818237), _V(1,0,0), (float)(360*RAD));
   
   //Rear left Wheel
   static UINT WGrp3[1] = {15};
   
   static MGROUP_ROTATE wheel2 (0,WGrp3, 1, _V(-1.24708,-1.54362,-0.3932629), _V(1,0,0), (float)(360*RAD));
   
   //Rear Right wheel
   static UINT WGrp4[1] = {14};
   
   static MGROUP_ROTATE wheel3 (0,WGrp4, 1, _V(1.24708,-1.54362,-0.3932629), _V(1,0,0), (float)(360*RAD));
   
   anim_wheel = CreateAnimation (0);
   
   //AddAnimationComponent  (anim_wheel, 0, 1, &wheel1);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel2);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel3);
   
   
   
   // Gear anims
  
   
   //Front Gear
   static UINT GGrp1[2] = {16,17};
   
   static MGROUP_ROTATE gear1 (0,GGrp1, 2, _V(0,-0.4173763,4.409041), _V(1,0,0), (float)(67*RAD));

   anim_gear = CreateAnimation (0);
   
   AddAnimationComponent  (anim_gear, 0, 1, &gear1);

  
   //Rudder
   static UINT RGrp1[1] = {23};
   
   static MGROUP_ROTATE rudderl (0,RGrp1, 1, _V(-5.912134,0.6803166,-2.658892), _V(0,1,-0.5), (float)(-18*RAD));
   
   static UINT LGrp1[1] = {25};
   
   static MGROUP_ROTATE rudderr (0,LGrp1, 1, _V(5.918832,0.6803166,-2.658892), _V(0,1,-0.5), (float)(-18*RAD));
   
   
   anim_rudder = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderl);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderr);
   
   
   //Elevator
   static UINT EGrp1[1] = {21};
   
   static MGROUP_ROTATE elevator (0,EGrp1, 1, _V(0,0.3209522,3.281828), _V(1,0,0), (float)(-45*RAD));
   
   static UINT EGrp3[4] = {52,53,54,55};
   
   static MGROUP_ROTATE elevator3 (1,EGrp3, 4, _V(0,6.957947E-02,0.2477936), _V(1,0,0), (float)(-45*RAD));
   
   
   // Stick Forward
   static UINT EGrp2[2] = {44,45};
   
   static MGROUP_TRANSLATE elevator2 (1,EGrp2, 2,  _V(0,0,-0.02));
   
   
   anim_elevator = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_elevator, 0, 1, &elevator);
   AddAnimationComponent  (anim_elevator, 0, 1, &elevator2);
   AddAnimationComponent  (anim_elevator, 0, 1, &elevator3);
   
   
   //Left Aleron
   static UINT LAGrp1[1] = {27};
   
   static MGROUP_ROTATE laileron (0,LAGrp1, 1, _V(-2.830395,0.083016,-1.62), _V(1,0,0.2), (float)(-33*RAD));
   
   
   //Steer 
   static UINT SLAGrp1[2] = {44,45};
   
   static MGROUP_ROTATE slaileron1 (1,SLAGrp1, 2, _V(0.1037424,-7.686364E-02,1.200089E-02), _V(0,0,1), (float)(-33*RAD));
   
   
   
   anim_laileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_laileron, 0, 1, &laileron);
   AddAnimationComponent  (anim_laileron, 0, 1, &slaileron1);
   
   
   
   //Right Aleron
   static UINT RAGrp1[1] = {19};
   
   static MGROUP_ROTATE raileron (0,RAGrp1, 1, _V(2.830395,0.083016,-1.62), _V(1,0,-0.2), (float)(33*RAD));
  
   anim_raileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_raileron, 0, 1, &raileron);
   
   
   // Prop Anims
   
   static UINT Propa1[1] = {64};
   
   static MGROUP_ROTATE propa1 (0,Propa1, 1, _V(-1.903685E-02,0.1247745,-2.66445), _V(0,0,1), (float)(360*RAD));
   
   
   
   static UINT Propa2[1] = {0};
   
   static MGROUP_ROTATE propa2 (2,Propa2, 1, _V(0,0,0), _V(0,0,1), (float)(360*RAD));
   
   
   anim_propelar = CreateAnimation (0);
   
   AddAnimationComponent  (anim_propelar, 0, 1, &propa1);
   AddAnimationComponent  (anim_propelar, 0, 1, &propa2);
  
   static UINT Propd1[1] = {65};
   
   static MGROUP_SCALE propd1 (0,Propd1, 1, _V(0,0,0), _V(0.001,0.001,0.001));
   
   anim_propscale = CreateAnimation (0);
   
   AddAnimationComponent  (anim_propscale, 0, 1, &propd1);
   
   
 
  // VSI Animations
   
   // Dial1
   static UINT dGrp0[1] = {64};
   
   static MGROUP_ROTATE dial1 (1,dGrp0, 1, _V(-9.849975E-02,6.358594E-02,0), _V(0,0,1), (float)(-222*RAD));
   
   
   anim_dial1 = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_dial1, 0, 1, &dial1);
   

    // Direction Dial Animations
   
   // Dial2
   static UINT d2Grp0[1] = {50};
   
   static MGROUP_ROTATE dial2 (1,d2Grp0, 1, _V(-9.431026E-02,2.100662E-02,0), _V(0,0,1), (float)(360*RAD));
   
   
   anim_dial2 = CreateAnimation (0.15);
   
   AddAnimationComponent  (anim_dial2, 0, 1, &dial2);

    

   // Throttle Animation
   
   static UINT Pow0[2] = {62,63};
   
   static MGROUP_ROTATE powg (1,Pow0, 2, _V(0,-5.434419E-02,5.973519E-02), _V(1,0,0), (float)(45*RAD));
   
   static UINT Pow2[1] = {61};
   
   static MGROUP_ROTATE powg2 (1,Pow2, 1, _V(-0.1198337,-7.673436E-03,0), _V(0,0,1), (float)(-200*RAD));
   
   
   anim_power = CreateAnimation (0);
   
   AddAnimationComponent  (anim_power, 0, 1, &powg);
   AddAnimationComponent  (anim_power, 0, 1, &powg2);
   
    
   // Speed Dial Animations
   
   // Dial3
   static UINT d3Grp0[1] = {6};
   
   static MGROUP_ROTATE dial3 (1,d3Grp0, 1, _V(-2.371191E-03,0.0802852,0), _V(0,0,1), (float)(360*RAD));
   
   
   anim_dial3 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial3, 0, 1, &dial3);

   
  
    // Altimeter Animations
   
   // Dial Long
   static UINT wWGrp0[1] = {8};
   
   static MGROUP_ROTATE wwheel0 (1,wWGrp0, 1, _V(-4.866757E-02,6.942312E-02,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_wheel0 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel0, 0, 1, &wwheel0);

   // Dial Short
   static UINT wWGrp1[1] = {7};
   
   static MGROUP_ROTATE wwheel1 (1,wWGrp1, 1, _V(-4.866757E-02,6.942312E-02,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_wheel1 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel1, 0, 1, &wwheel1);
   
   
   
   // Horizon Bank Guage Animations with new Hor2 Ball Pitch Anims
   
   static UINT BAGrp0[1] = {67};
   
   static MGROUP_ROTATE bank2 (1,BAGrp0, 1, _V(-4.845047E-02,2.742418E-02,6.320879E-02), _V(0,0,1), (float)(-360*RAD));
   
   
   static UINT PAGrp1[1] = {56};
   
   pitch1 = new MGROUP_ROTATE (1,PAGrp1, 1, _V(-4.845047E-02,2.742418E-02,6.320879E-02), _V(1,0,0), (float)(-360*RAD));
   

   
   anim_pitch = CreateAnimation (0.5);
   anim_bank = CreateAnimation (0.5);
   
   parent1 = AddAnimationComponent  (anim_bank, 0, 1, &bank2);
   AddAnimationComponent  (anim_pitch, 0, 1, pitch1, parent1);
   
   
   //Canopy
   static UINT CGrp1[3] = {0,1,67};
   
   static MGROUP_ROTATE can1 (0,CGrp1, 3, _V(0.3939091,0.4020558,1.008745), _V(0,0,1), (float)(-90*RAD));
   
   
   anim_canopy = CreateAnimation (0);
   
   AddAnimationComponent  (anim_canopy, 0, 1, &can1);

   
    //Front Gear turn
   static UINT TuGrp1[1] = {17};
   
   static MGROUP_ROTATE turn1 (0,TuGrp1, 1, _V(0,-1.202334,3.991869), _V(0,1,0), (float)(90*RAD));

   anim_turn = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_turn, 0, 1, &turn1);
   
}
	

bool ShuttlePB::clbkLoadVC (int id)
{
	
	SetCameraOffset (_V(0,0.02,0));
	
    SetCameraDefaultDirection (_V(0,0,1));

    SetCameraRotationRange (RAD*25, RAD*25, RAD*20, RAD*20);	
	
	
 return true;

}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new ShuttlePB (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (ShuttlePB*)vessel;
}
